DrawLocation
; draw a single game location on the screen
;HL: room data address	

;store start of current room data
	LD (CurrentLocation),HL		
		
;clear screen for new location	
	PUSH HL
	CALL ClearLocation	
	
;Clear animated objects data by default
	XOR A	
	LD (TextEnterLocation),A	;by default no message is displayed when entering a new location
	LD (NeonStatus),A
	LD (FlamesStatus),A
	LD (ShootHereStatus),A
	LD (BleedingStatus),A
	LD (GasStatus),A
	
	LD A,255
	LD (GameCycle),A			;start cycles from 0, 255 will be increased
	
;Room title
;draw  stripe in the text line 
	LD HL,16544
	LD DE,Border01
	LD BC,8193;B=32,C=1
	CALL TileBlock
	
;print room title	
	POP HL			; restore room data address
	LD A,(HL)		; read position of title
	INC HL
	PUSH AF			; store position
	LD A,(HL)
	LD E,A
	INC HL
	LD A,(HL)	
	LD D,A			; DE loaded with text address	
		
	
	POP AF			; restore position
	PUSH HL			; store room data address
	LD H,64
	LD L,A 			; position of room title
	CALL TypeText
	POP HL			; restore room data address
	
	INC HL			; go to next room data
	
	EI	
	HALT
	DI

;1.Draw tile blocks	

;Each block is stored with 4 bytes
;Byte 1: number of tile
;Byte 2,3: size (B-width,C-height) of block 
;Byte 4:  L coordinate of left upper corner
	;(the whole address is stored IN HL, but H can have only 2 values (we use 2 screen segments), so to save memory
	;H isn't stored and tile drawing loop is done twice  with  H=72 and H=80		
		
		LD A,72
		LD (SecondOrThirdSegment),A			
		
LocationBlocks1
		LD A, (HL)						;number of tile	loaded in A
		OR A
		JR Z, LocationBlocks2			;if (HL)=0 that means either end of drawing blocks
										;or going to next of screen segments
										;(increasing H by 8)
		
		CALL CountTileAddr				;count address of tile in memory and put it into DE
		
		INC HL 
		LD A,(HL)
		LD B,A
		INC HL
		LD A,(HL)
		LD C,A							;BC is loaded with block size
				
		INC HL
		PUSH HL							;HL containing  room data address stored
		
		LD A,(HL)			
				
		LD L,A			
		LD A, (SecondOrThirdSegment)	
		LD H,A							;HL is loaded with block left upper corner
		
		
		CALL TileBlock					;draw block of tiles
		
		POP HL							;restore room blocks data address 
		INC HL							;start of next block data
		JR LocationBlocks1
		
SecondOrThirdSegment	.BYTE 72	

LocationBlocks2
;go to another screen block or end  blocks drawing	

		INC HL							;go to next byte of room data
		LD A,(SecondOrThirdSegment)
		CP 80							;check if it is last screen segment
		JR Z, LocationTiles1 				;if so go to single tiles drawing
		ADD A,8							;if not go to another screen segment
		LD (SecondOrThirdSegment),A
		JR LocationBlocks1

		
;2. Draw single tiles

;Tiles are stored with 2 bytes
	;1st byte: number of tile
	;2nd byte: L of HL storing left upper tile corner (H is 72 or 80)
		
LocationTiles1
		LD A,72
		LD (SecondOrThirdSegment),A
		
LocationTiles2
		LD A, (HL)						;number of tile	loaded in A
		
		OR A
		JR Z, LocationTiles3			;if (HL)=0 that means either end of drawing blocks
										;or going to next of three screen segments (increasing H by 8)
		
		CALL CountTileAddr				;count address of tile in memory and put it in DE

		INC HL
		PUSH HL							;store room data address
		LD A, (HL)
		LD L,A
		LD A, (SecondOrThirdSegment)
		LD H,A							;put upper left tile corner in HL
						
		CALL DrawTile					;call  routine drawing single tile
		
		POP HL							;restore room data address
		INC HL							;go to next room data byte
		JR LocationTiles2
					
LocationTiles3
		INC HL							;go to next byte of room data
		LD A,(SecondOrThirdSegment)
		CP 80							;check if last screen part
		JR Z, LocationAttrs1 				;if so end drawing tiles
		ADD A,8							;if not, go to another screen segment
		LD (SecondOrThirdSegment),A
					
		JR LocationTiles2

		
;3. Attributes
		;Fills rectangular blocks with given attribute
		;Attribute blockss are stored with 4 bytes
			;1st byte: 			attribute
			;2nd and 3rd byte: 	left upper block corner address
			;4th and 5th byte: 	attr block size
					
LocationAttrs1 
		LD A,(HL)						;load attribute from memory
		OR A							;0 means end of writing attributes 		
		JR Z,LocationMirror1
		PUSH AF
		INC HL							;write screen address to DE
		LD A,(HL)
		LD D,A
		INC HL
		LD A,(HL)
		LD E,A
		INC HL							;write attributes block size to BC
		LD A,(HL)
		LD B,A							;width
		INC HL
		LD A,(HL)
		LD C,A							;height
		INC HL
		POP AF							;restore attribute
		CALL AttrBlock					;draw attributes
		
		JR LocationAttrs1

;4.Mirror some areas of screen		

;Mirrors are stored with 4 bytes:		
	;1st and 2nd byte:  position of mirrored area 
	;3rd and 4th byte:  size of mirrored area in characters
	
LocationMirror1		
	INC HL
	LD A,(HL)
	OR A								;0=end of mirrored areas
	JR Z,LocationNeighbours
	LD D,A
	INC HL
	LD A,(HL)
	LD E,A								;mirror position in DE
	INC HL
	LD A,(HL)
	LD B,A
	INC HL
	LD A,(HL)
	LD C,A								;size in BC
	PUSH HL
	EX DE,HL							;mirror position in HL
	CALL MirrorArea 					;make mirror
	POP HL		
	JR LocationMirror1	

	

;Drawing room is finished
;but we must read some room parameters
	
		
;5. Set neighbouring rooms		
LocationNeighbours
	INC HL
	LD E,(HL)	
	INC HL
	LD D,(HL)		
	LD (LeftEdgeLocation),DE 			;left neighbour stored 

	INC HL
	LD E,(HL)	
	INC HL
	LD D,(HL)		
	LD (RightEdgeLocation),DE			;right neighbour stored

; 6. Room code	
;InitRoomCode contains address of code which is executed after drawing room
;this code is responsible for special effect in this room
	INC HL
	LD E,(HL)
	INC HL
	LD D,(HL)
	LD (InitRoomCode),DE; store room code

; 7. Events
;Current Events contain address of events data. Events happen after pressing fire
	INC HL
	LD (CurrentEvents),HL
		
; 8. Executing room code
	
	LD A,D
	OR E					; if room code=0, do not execute room code
	RET Z
	EX DE,HL				; load room code address IN HL
	JP (HL)					;go to room code, execute it and after RET in room code subroutine end of drawing room
	
;room drawing is finished
	
	
;------------------------------
CountTileAddr	
;exchange one byte tile number in A to tile address in DE
		;A-tile number
		;DE- tile address as routine result
	PUSH HL
	LD HL,TileAddrData-2
	LD E,A
	LD D,0
	ADD HL,DE				;HL=HL=2*A
	ADD HL,DE				;HL points to tile address stored in memory
	
	LD E,(HL)
	INC HL
	LD D,(HL)				;sprite address placed in DE		
	POP HL	
	RET

	
;-----------------------------------------------------------------------------
TileBlock
;draw a block of tiles (one tile used)
	;BC: block size (B - width, C - height)
	;DE: tile address
	;HL: left upper corner of the block	

		
TileBlock1
		PUSH BC
		PUSH DE
		PUSH HL

;draw one horizontal line of tiles
TileBlock2		
		PUSH BC
		PUSH DE
		PUSH HL
		CALL DrawTile		; draw a single tile

		POP HL
		POP DE 				; restore screen position and tile address
	
		LD A,(DE)
		LD B,A 				; read tile width from memory and write to B
	
		LD A,L
		ADD A,B
		LD L,A				; count horizontal coordinate of next tile in the line
	 						;(add B=tile width to coordinate of previous tile)
			
		POP BC				;restore block size
					
		DJNZ TileBlock2		;repeat until all line of tiles is drawn
		
		
; draws C lines of tiles
		POP HL
		POP DE
	
		INC DE
		LD A,(DE)			;read tile height from memory
		LD C,A				;write tile height to C	
		DEC DE	
	
		LD B,L				;B is used to check carry		
		LD A,L	
TileBlock3	
		ADD A,32 			;go C lines down,because C=tile height
		DEC C
		JR NZ, TileBlock3
		LD L,A				
		
;check if new tile begins in next screen segment
;(carry happens when L<B(number exceeded 256)	
		SUB B	
		JR NC, TileBlock4	;if no carry, just do nothing
		
		LD A,H
		ADD A,8
		LD H,A				;if carry change address properly
		
				
TileBlock4
		POP BC				; restore block size in BC
		DEC C				; decrease number of tiles in collumn yet to do
		JR NZ, TileBlock1
		RET
		
;-----------------------------------------		
AttrBlock
;fill rectangular area with attribute
	;BC: area size B-width, C-height
	;DE:area left upper corner
	;A: attribute	
	
		;PUSH BC		;check if ink set to black, if so do not change ink on screen
		;LD B,A
		;AND 7
		;LD A,B
		;POP BC
		;JR Z,AttrBlock4
		
;draw ink option				
AttrBlock2
		PUSH BC
		PUSH DE
AttrBlock3
		LD (DE),A
		INC DE 		;go to next square in the row
		DJNZ AttrBlock3
		
		POP DE
		POP BC
		PUSH HL ;next line of attributes
		LD HL,32 ;(DE=DE+32)
		ADD HL,DE 
		EX DE,HL
		POP HL
	
		DEC C ;decreases number of lines yet to do 
		
		JR NZ,AttrBlock2
		RET
		
;no draw ink option
AttrBlock4
		PUSH AF
		PUSH BC
		PUSH DE
		LD C,A;stores attribute IN C
AttrBlock5	
		LD A,(DE)
		AND 7;extract ink
		ADD A,C;combine with attribute
		LD (DE),A
		INC DE ;goes to next square IN the row
		DJNZ AttrBlock5
		POP DE
		POP BC
		
		PUSH HL ;next line of attributes
		LD HL,32 ;(DE=DE+32)
		ADD HL,DE 
		EX DE,HL
		POP HL
		POP AF;restore attribute
		
		DEC C ;decreases number of lines yet to do 
		
		JR NZ,AttrBlock4
		RET

;--------------------------------------
;draw a single tile  on the screen
	;HL: upper left tile corner size	
	;BC: tile size (not a parameter but read by this routine from tile data)
	;DE: tile memory address	

;FIRST BYTE OF 8 BYTES IN A CHAR CANNOT BE 16, BECAUSE PROGRAM WILL INTERPRET IT AS AN EMPTY SQUARE!!!
		
DrawTile

	LD A,(DE)				;load tile size to BC
	LD B,A 					;x-width
	INC DE
	LD A,(DE)
	LD C,A 					;y-height	
	INC DE			
				
	PUSH HL					;store position
	PUSH BC					;store size
			
	LD(StoreStack),SP		;SP=DE
	EX HL,DE				;stack points to tile graphics
	LD SP,HL
	EX HL,DE

	LD IXH,B
	LD IXL,C
	
DrawTile1	
	LD C,H				;store H for top of characters in the current row
	LD A,L				;store L
	
	LD B,IXH			;number of characters in the row

DrawTile2
	POP DE				;read 2 bytes of tile graphics
	EX AF,AF'			;work with alternate A
	LD A,E
	CP 16				;16 codes empty square
	JR NZ,DrawTile3		;normal, not empty character
		
	DEC SP				; POP DE increased SP by 2 and we skip only 1 byte, so decrement SP
	JR DrawTile4		;empty char, skip data drawing
		

DrawTile3
		
;draw 8 bytes, one character
		;POP DE was already done
		LD (HL),E
		INC H
		LD (HL),D
		INC H
		
		POP DE				;draw two bytes
		LD (HL),E
		INC H
		LD (HL),D
		INC H
		
		POP DE				;draw two bytes
		LD (HL),E
		INC H
		LD (HL),D
		INC H
		
		POP DE				;draw two bytes
		LD (HL),E
		INC H
		LD (HL),D
	
DrawTile4
		EX AF,AF'			;back to normal A again		
		LD H,C				;restore H
		INC L				;go right to next character
		DJNZ DrawTile2		;repeat until whole row of characters is drawn
		
;restore leftmost character of the row and go one char down
		LD H,C				;restore H
		ADD A,32			;A stored L
		LD L,A				;now L=L+32
		JR NC,DrawTile5

;change H only if going to next screen segment	
			LD A,H
			ADD A,8
			LD H,A
					
DrawTile5	
		DEC IXL
		JR NZ,DrawTile1
		
		
		LD HL,0				;DE points to attributes now
		ADD HL,SP
		EX HL,DE
		
		LD SP,(StoreStack)	;restore old stack		
	
		
			
		
; graphics are ready, now paint attributes
; DE is already increased to next byte where attributes data begin
						
		POP BC
		POP HL		
							
;ATTRIBUTES
	;BC: tile size (B-width C-height)
	;DE:start of attributes data		
	;HL: screen address of left upper tile corner				
TileAttrs		
		LD A,80
		SRL H
		SRL H
		SRL H
		ADD A,H
		LD H,A ;transform graphics address to attribute address
			
TileAttr1
	 	PUSH BC
		PUSH HL
		
;draw one row of attributes
TileAttr2
	    LD A,(DE)
		OR A				;check if attribute is set to 0
		JR Z, TileAttr3		; if so, don't paint attribute and leave the square as it is
		LD (HL),A			; paint attribute
TileAttr3
		INC HL 				;go to next square in the row
		INC DE
		DJNZ TileAttr2
	
		POP HL
		PUSH DE  
		LD DE,32 
		ADD HL,DE			;go to next row of attributes
		POP DE 	 
		POP BC
		DEC C 				;decrease number of rows left to do 
		JR NZ, TileAttr1
		RET					;end of tile drawing


;-----------------------------------

MirrorArea
;make mirror of graphic area in the screen
	;HL: upper left corner of mirrored area
	;BC: area size
	;DE: doesn't contain any parameters but is used
	
		
;The routine has to match to every character in the area to the opposite mirror character
;it has to count it address AND place it IN DE using initial char address placed IN HL
; vertical coordinate is easy, it is just D=H
; there is more trouble with counting E
; we should use the equation:
; E= 2*l0 + B -1 -L
; where l0 - horizontal coordinate of leftmost char IN row 
; B - horizontal size of the area (stored IN B
; but if we put L into equation as l0 we have its simpler version:
; E=2*L+B-1-L=L+B-1
		

	PUSH HL
	PUSH BC ;stores	mirror block data for further use IN attribute mirror routine	
		
		
MirrorArea1
		PUSH HL
		PUSH BC		
			

;counts E					
			LD A,L		; A=l0
			ADD A,B		; A=l0+B
			DEC A			; A=l0+B-1
			LD E,A		; E=l0+B-1
			LD D,H
	
;if B is even (E.g. 8)the loop should be egzecuted B/2 times(4)
;if B is odd  (E.g. 11)the loop should be egzecuted B/2+1 times(6)

			
			XOR A;A=0
			SRL B
			ADC A,B
			LD B,A	
		
;mirrors one line of characters
MirrorArea2
		PUSH BC
		PUSH HL
		PUSH DE
			
		LD C,8
		

;mirrors 2 characters
MirrorArea3
				LD A,(HL)	
				LD B,0
				RLCA
				RR B
				RLCA
				RR B
				RLCA 
				RR B
				RLCA
				RR B
				RLCA
				RR B
				RLCA
				RR B
				RLCA
				RR B
				RLCA
				RR B		;byte with address in HL is mirrored and stored in B
				
				LD A,(DE)
				PUSH BC
				LD C,0
				RLCA
				RR C
				RLCA
				RR C
				RLCA
				RR C
				RLCA
				RR C
				RLCA
				RR C
				RLCA
				RR C
				RLCA
				RR C
				RLCA
				RR C
				LD A,C
				POP BC		;byte with address in DE is mirrored and stored in A
				
				
				LD (HL),A
				LD A,B
				LD (DE),A 	;mirrored HL and DE bytes are swapped
				INC H
				INC D
				
				DEC C
				JR NZ, MirrorArea3
	
		POP DE
		POP HL
		INC L
		DEC E		; goes to another pair of chars
		
		POP BC
		DJNZ MirrorArea2
		
	
	POP BC
	POP HL	
	
;goes to another line of chars	
	LD A,L
	ADD A,32	
	LD L,A		
	
;checks if mirrored line goes to next screen segment
	JR NC, MirrorArea4 		;if not=no carry, do nothing
	LD A,H
	ADD A,8
	LD H,A 					; if so, change address properly
		
MirrorArea4
		DEC C				; decreases number of lines yet to do			
		JR NZ, MirrorArea1	; begin routine from the start for another line

;Now it its time to mirror attributes
	
		POP BC
		POP HL
		
		LD A,80
		SRL H
		SRL H
		SRL H
		ADD A,H
		LD H,A ; counts address of first attribute char

		;counts address for opposite char (to HL char)IN the line
		;AND places it IN DE				
		
MrAttr1	PUSH BC
		PUSH HL
		
		LD A,L		; A=l0
		ADD A,B		; A=l0+B
		DEC A			; A=l0+B-1
		LD E,A		; E=l0+B-1
		LD D,H		;HL is mirrored IN DE now
		
		SRL B ;B is Divided by two
			;Mirrored block is B chars wide
			;but only B/2 chars attributes swaps are necessary
			; IN case B is even (E.g. 6) all attributes are symetrically swapped
			;IN case B is odd (E.g. 11) all are swapped except for the middle char
			;which doesnt need attribute change
		
MrAttr2	LD A,(HL)
		LD C,A	;HL attribute is placed IN C which is free
		LD A,(DE)
		LD (HL),A	;(DE) is placed IN (HL)
		LD A,C
		LD (DE),A	;(HL) is placed IN (DE)
		
		INC L
		DEC E	;go to another par of attributes
		 
		DJNZ MrAttr2 ; decreases B AND repeats routine until the whole line is swapped
		
		POP HL
		POP BC
		
		PUSH DE
		LD DE,32
		ADD HL,DE;go to next line
		POP DE	
		
		DEC C ; number of lines yet to do
		JR NZ, MrAttr1; repeat the whole routine for the new line
		
		RET


;----------------------------------------
;refresh current room (draw again current room after item action)	
RefreshCurrentLocation		
	LD HL,(CurrentLocation)
	CALL DrawLocation
	CALL DrawItems
	
	
;draw player and store new background, forgetting about background from room left
	LD HL,(OldPlayerPos)	
	CALL DrawPlayer	
	
	RET
;----------------		
GoIntoNewLocation
;DE address of room data
;BC position of player in new room
	LD A,20
GoIntoNewRoom2
	EI
	HALT
	DI
	DEC A
	JR NZ,GoIntoNewRoom2
	
;draw new room
	EX DE,HL; HL loaded with room address
	PUSH BC
	CALL DrawLocation
	CALL DrawItems
;draw player and store new background, forgetting about background from room left
	
	;update position
	POP BC
	LD (OldPlayerPos),BC
	LD (NewPlayerPos),BC
	
;store background and draw player	
	LD H,B
	LD L,C ;HL loaded with player position
	
	CALL DrawPlayer
		
;should a message be displayed when entering the location	
	LD A,(TextEnterLocation)
	OR A
	RET Z
	CALL TypeMessage		;displaying text
	
	RET

;------------------
ClearLocation
	EI
	HALT
	DI
	LD (StoreStack),SP

	
;clear bottom 2/3 of screen
	LD DE,0
	LD SP,16384+6144
	LD B,128
ClearLocation3
	REPT 16
	PUSH DE
	ENDR
	DJNZ ClearLocation3
	
;white attributes
	LD SP,22528+768					;end of attributes 
	LD B,32 						;clear 32*8 PUSH DE*2 bytes per DE=512 bytes (16 lines)
	LD DE,71*256+71					;white
ClearLocation4
	REPT 8
	PUSH DE
	ENDR		
	DJNZ ClearLocation4
	
	LD SP,(StoreStack)
	RET

;------------------
ClearScreen
;clear whole screen
	XOR A
	OUT (254),A
	LD HL,16384
	LD DE,16385
	LD BC,6143
	LD (HL),A
	LDIR
	
	LD A,71
	LD HL,22528
	LD DE,22529
	LD BC,767
	LD (HL),A
	LDIR
	RET