
;----------STAND------------------------
	;draws player standing

StandInPlace		
	;if player was walking,change walking flag  to 0
	XOR A
	LD (Walking),A	
		
; mask
	LD HL,MaskBuffer
	LD DE,RonStandM			;mask data	
	LD C,48					;number of lines
	CALL MaskIntoBuffer2

	LD A,(Rotation)
	LD C,A
	CALL RotateMask
	
;Prepare standing sprite in buffer	

	LD HL,SpriteBuffer
	LD DE,RonStand; sprite data	
	LD C,48; number of lines
	CALL SpriteIntoBuffer2

	LD A,(Rotation)
	LD C,A
	CALL RotateSprite
		
	;Start with new screen interrupt
	EI
	HALT	
	DI
	;Restore Background
	LD HL,(OldPlayerPos);screen address of  background being restored
	CALL DrawBackground
	
	;Draw standing sprite	
	LD HL, (NewPlayerPos)	
	CALL DrawPlayer	
	
	LD HL, (NewPlayerPos)	
	LD (OldPlayerPos),HL
	
	;Draw animated objects in foreground if necessary
	CALL CheckFrontAnimations
	RET
	
	
;--------------------	
FirstStandInPlace
;called only once at the beginning of the game

;Prepare standing mask
	LD HL,MaskBuffer
	LD DE,RonStandM; mask data	
	LD C,48; number of lines
	CALL MaskIntoBuffer2
	
;Prepare standing sprite	
	LD HL,SpriteBuffer
	LD DE,RonStand; sprite data	
	LD C,48; number of lines
	CALL SpriteIntoBuffer2
	
;Draw standing sprite
	LD HL, (OldPlayerPos)	
	CALL DrawPlayer	
	RET
	
	
;---------PREPARE SPRITE IN BUFFER-------------
		
PreparePlayerSprite
	
;prepare (put into buffer) player sprite number B  rotated C pixels right
;parameters:
; B - number of sprite frame 
; C - rotation  
	LD DE,PlayerSpriteAddrs-4; 

;count mask address using table of addresses and sprite number	
CountMaskAddr
	INC DE
	INC DE
	INC DE
	INC DE	
	DJNZ CountMaskAddr	;repeat B times,after this loop DE contains 
						;place in memory where address of mask is stored
			
	PUSH DE				;store this address to be used when couting sprite address
		
	LD A, (DE)
	LD B,A
	INC DE
	LD A, (DE)
	LD E,B
	LD D,A	;now DE contains mask address
			
;put  mask into buffer
	PUSH BC
	CALL MaskIntoBuffer 
	POP BC
			
;count sprite address
	POP DE;restore mask address
	
	INC DE
	INC DE	;now DE contains place in memory
			;where address of sprite is stored
	LD A, (DE)
	LD B,A
	INC DE
	LD A, (DE)
	LD E,B
	LD D,A	;now DE contains sprite address
		
	;put  sprite into buffer	
	CALL SpriteIntoBuffer
	RET

	
;--------------------
MaskIntoBuffer
;DE - adress of mask data
;C - rotation
;both player mask and sprite are built of player head which doesn't change
;and player body which has several frames
	PUSH BC; store rotation
	
;write mask of player head into buffer
	PUSH DE								;store address of body mask
	LD DE,RonHeadM
	LD HL, MaskBuffer					;head mask buffer 
	LD C,16								;head - number of lines	
	CALL MaskIntoBuffer2				;puts head mask in buffer
	POP DE

	
;write mask of player body into buffer		
	LD HL, MaskBuffer+64				;body mask buffer 
	LD C,32								;body - number of lines
	CALL MaskIntoBuffer2 				;put body mask in buffer	
		
	POP BC; restores rotation
	CALL RotateMask	
	RET

;-------------		

RotateMask
;mask is in buffer now it is time to rotate it
	LD A,C
	OR A
	RET Z								;if no rotation required , just return
	
;modify self-modifying code
	LD HL,Write1or0
	LD (HL),0							;when mask is rotated, the left side of buffer is filled with lit pixels
			 							;which is made by code SCF NOP (0 means NOP)
			
RotateMask1
		LD HL,MaskBuffer				;start of rotated mask data
		CALL RotateBuffer				;make one rotation
		DEC C
		JR NZ, RotateMask1				;make C rotations	
		RET	
	
;-------------		
MaskIntoBuffer2		
;Write mask into buffer		
;Parameters:
 	;DE - start of mask data    
	;HL - start of buffer
	;C - number of lines to be put into buffer
	
	LD B,3								;original mask size (without borders) - number of bytes per line
		
MaskIntoBuffer3 
		PUSH BC		
MaskIntoBuffer4
		 	LD A,(DE) 
			LD (HL),A;write one line to work area
		 	INC L						;next buffer address (no need of doing INC HL)
										;cause there is no posibility of carry)
		 	INC DE
		 	DJNZ MaskIntoBuffer4
		
		LD (HL),255						;empty column filled with lit pixels - right side of mask
		INC L
		POP BC
		DEC C 							;decrease number of lines yet to do
		JR NZ, MaskIntoBuffer3
		RET

		
;--------------------
SpriteIntoBuffer
	;DE - adress of sprite data
	;C - rotation
	
	PUSH BC; store rotation
;write sprite of player head into buffer
	PUSH DE								;store address of sprite
	LD DE,RonHead
	LD C,16								;head - number of lines	
	LD HL, SpriteBuffer					; head sprite buffer 
	CALL SpriteIntoBuffer2 				;put head sprite in buffer
	POP DE
	
	
	
;write sprite of player body into buffer	
	LD HL, SpriteBuffer+64				;body sprite buffer 
	LD C,32								;body - number of lines
	CALL SpriteIntoBuffer2				;puts body sprite in buffer	
		
	POP BC; restoring rotation
	CALL RotateSprite
	RET
	
		
;----------------
RotateSprite
;sprite is in buffer, now it is time to rotate it
		LD A,C
		OR A ;check if rotation necessary
		JR NZ, RotateSpriteRequired
		EI
		HALT ;delay when not rotating
		DI
		RET

RotateSpriteRequired	
;modify self-modifyng code
	LD HL,Write1or0			;when mask is rotated, the left side of buffer is filled with empty pixels
	LD (HL),63				;which is made by code SCF CCF (63 means CCF)
		
RotateSprite1
	LD HL, SpriteBuffer	
	CALL RotateBuffer		;make one rotation
	DEC C
	JR NZ,RotateSprite1		;make C rotations
	RET
		
;-----------------
;write sprite into buffer	
SpriteIntoBuffer2
		
; Write sprite into work buffer		
; This routine should be called with
 	; start of sprite data  in DE  
	; start of buffer in HL
	; number of lines to be put into buffer in C
	
	
	LD B,3; sprite size - number of bytes per line
	
SpriteIntoBuffer3
		 PUSH BC		
		
SpriteIntoBuffer4
		 LD A,(DE) 
		 LD (HL),A;write one line to buffer
		 INC L; again no INC HL cause no danger of carry
		 INC DE
		 DJNZ SpriteIntoBuffer4
		 LD (HL),0; empty column- right side of sprite
		 INC L
		 POP BC
		 DEC C ;decreases number of lines yet to do
		 JR NZ, SpriteIntoBuffer3
		 RET
		
;--------------		
RotateBuffer
;Rotate one pixel right the data in buffer
;can be called both for mask and sprite
;HL = start of rotated data placed in buffer
		LD B,48; number of lines
	
RotateBuffer1	
			SCF 
Write1or0	CCF 
			; When rotating sprite carry flag should be set to 0 because we put empty space left of sprite
		    ; but when rotating mask it should be set to 1 because only center of mask is empty and its borders are filled with lit pixels		
		    ; so Write1or0 cell should contain one of both:
		    		; NOP command which leaves carry as 1 (set by previous command SCF)
					; CCF command which changes it to 0	 
		
		RR (HL)	;rotate right one byte
		INC L
		RR (HL)
		INC L
		RR (HL)
		INC L
		RR (HL)
		INC L
;rotated sprite which is 4 chars wide, for any other size the routine should be changed	
	DJNZ RotateBuffer1; number of lines yet to rotate
	RET
	
	
	
;
	RET


	
	
;--------PREPARE JUMP SPRITE-------
JumpSpriteIntoBuffer

;put into buffer jumping mask and sprite and rotate it
	
	;put in buffer jumping mask 
	LD DE,RonHeadM			
	LD HL,MaskBuffer
	LD C,16
	CALL MaskIntoBuffer2
	
	LD DE,RonJumpM			
	LD HL,MaskBuffer+64
	LD C,32
	CALL MaskIntoBuffer2
;rotation
	LD A,(Rotation)
	LD C,A
	CALL RotateMask
		
			
;put in buffer jumping sprite
	LD DE,RonHead
	LD HL,SpriteBuffer
	LD C,16
	CALL SpriteIntoBuffer2

	LD DE,RonJump
	LD HL,SpriteBuffer+64
	LD C,32
	CALL SpriteIntoBuffer2
;rotation
	LD A,(Rotation)
	LD C,A
	CALL RotateSprite
	RET

		
		
;----------MIRRORING PLAYER SPRITES---------------------
MirrorPlayer
;this routine changes all player sprites and mask into their mirror images
;it assumes they are 3 bytes wide
		

	LD HL, RonStandM	;start of player data
	LD BC, 448	;number of lines (2*16+2*48+10*32)

MirrorPlayerLine
		LD A,(HL)
		CALL MirrorByte
		LD E,A 		;rotate left byte and store it in E
		
		INC HL
		LD A,(HL)
		CALL MirrorByte
		LD (HL),A 	;rotate middle byte and write it
		
		INC HL
		LD A,(HL)
		CALL MirrorByte
		LD D,A 		;rotate left byte and store it in D
		
		LD A,E
		LD (HL),A	;write left byte to right byte
		DEC HL
		DEC HL
		LD A,D	
		LD (HL),A	;write right byte to left byte
		
		INC HL
		INC HL
		INC HL		; go to next next 3 bytes line		
		
		DEC BC
		LD A,B
		OR C
		JR NZ,MirrorPlayerLine
		RET

;---------------				
MirrorByte	
		LD D,0
		RLCA
		RR D
		RLCA
		RR D
		RLCA
		RR D
		RLCA
		RR D
		RLCA
		RR D
		RLCA
		RR D
		RLCA
		RR D
		RLCA
		RR D
		LD A,D
		RET
		
		
;---------------CHANGE ROOM--------------------
;Routines changing room when player reaches screen edge
;HL - player position


GoNextScreenLeft
	LD A,L		; new player position L loaded in A
	ADD A,28	; it was collumn 0 and now it has to be collumn 28(right edge-4)
			 	; player was at the left edge, now he is at the right edge
	LD L,A
	LD (OldPlayerPos),HL
	LD (NewPlayerPos),HL ; change position
	
	
	;Draw new location
	PUSH HL
	LD HL,(LeftEdgeLocation)
	CALL DrawLocation
	CALL DrawItems
	POP HL
	
	;draw player and store new background, forgetting about background from previous room
	CALL DrawPlayer
	;should a message be displayed when entering the location	
	LD A,(TextEnterLocation)
	OR A
	RET Z
	CALL TypeMessage		;displaying text
	
	RET	

;---------
GoNextScreenRight
	LD A,L 	; new player position L loaded in A
	SUB 29 	; it was collumn 29 and now it has to be collumn 0 
		   	; player was at the right edge, now he is at the left edge	
	LD L,A
	LD (OldPlayerPos),HL
	LD (NewPlayerPos),HL ; change position

	;Draw new room	
	PUSH HL
	LD HL,(RightEdgeLocation)
	CALL DrawLocation
	CALL DrawItems
	POP HL	
	
;draw player and store new background, forgetting about background from previous room	
	CALL DrawPlayer
	;should a message be displayed when entering the location	
	LD A,(TextEnterLocation)
	OR A
	RET Z
	CALL TypeMessage		;displaying text
	RET	
	
	
		
		
;-----------DRAW PLAYER------------------
DrawPlayer	
;draw player at the position stored in HL
;at the same time store the background on which player is drawn into buffer

	LD DE, BackgroundBuffer		;start of background buffer 
	LD B,48						;height in lines
	EX HL,DE					;buffer in HL, screen in DE	

;draw one line of player sprite
DrawPlayer1
		LD C,E
	  	
		REPT 4			;repeat 4 times (4 bytes in line)
;Background
		LD A,(DE)				;background to A
		LD (HL),A				;store background to buffer
		INC H					;go to mask buffer
;Mask
		AND (HL)				;combine it with background
;Sprite
		INC H					;go to sprite buffer
		OR (HL)					;combine mask+background with sprite
		LD (DE),A				;and write it to background
		
		INC E					;next collumn
		INC L					;next data byte (no possibility of carry, so no need INC HL)
		DEC H					;back to background buffer	
		DEC H	
		
		ENDR					;end of repeat

	LD E,C						;restore leftmost byte of the line
	CALL OnePixelLineDownDE		;go to next line	
	DJNZ DrawPlayer1			;decrease number of lines left to do
	
	RET
	
	
;---------------------------------------
DrawBackground
;restore background at position stored in HL

	LD DE, BackgroundBuffer		;start of background data 
	LD B,4						;width in bytes
	LD C,48						;height in lines


;draw one line of background

DrawBackground1
;(routine can be called here if we want to draw anything other that a background from buffer)
	PUSH BC
	PUSH HL
DrawBackground2
		LD A,(DE)				;read background
		LD (HL),A				;and write it to screen
		
		INC L					;next collumn
		INC E					;next background byte (no possibility of carry, so no need INC DE)
		DJNZ DrawBackground2
	
	POP HL
	POP BC
	
	CALL OnePixelLineDown		;go to next line

DrawBackground3	
	DEC C 						;decrease number of lines left to do
	JR NZ,DrawBackground1
	RET	
	
;--------------
OnePixelLineDown
;requires HL as screen address

	INC H 								;go to new line
	LD A,H
	AND 00000111B						;if last three bits of h are 000 -> going to new character line
	RET NZ								;new line belongs to the same character line
	
	LD A,L 								;going to new character line
	ADD A,32
	LD L,A
	RET C								;going to new screen segment, if e.g h was 71, now it is 72
	
	LD A,H
	SUB 8
	LD H,A								;decrease h if we are in the same screen segment
	RET

;--------------	
OnePixelLineDownDE	
;the same but using DE register
	INC D
	LD A,D
	AND 00000111B
	RET NZ
	LD A,E
	ADD A,32
	LD E,A
	RET C
	LD A,D
	SUB 8
	LD D,A
	RET	


;---------------
OnePixelLineUp
;requires HL as screen address
	DEC H 								;go to new line
	LD A,H
	AND 00000111B
	CP 7								;h=0 -> going to new character line
	RET NZ								;new line belongs to the same character line
	
	LD A,L 								;going to new character line
	SUB 32
	LD L,A
	RET C	
	
	LD A,H
	ADD A,8
	LD H,A								;increase h if we are in the same screen segment
	RET