;---------TEXT-----------
TypeText	
;HL; contains position of text on screen
;DE contains text address in memory
;BC is used
		
;the text is compressed,each letter only takes 5 bytes (32 combinations) 
;for example AAA text is stored without compression as 1,1,1
;after compression it will take 2 bytes and will look in binary

; 00001|000  01|0|00001 
;and in decimal 8,65
;the bit in the middle of second byte is not used

	LD (TextLineAddress),HL
	
	LD A,71					;Default text color is white on black
	LD (TextColor),A
	XOR A				
	LD (LetterNr),A	
		
TypeText1	
	PUSH HL
	CALL DepackLetter		;read letter from compressed data and put it in A
	POP HL
	
	OR A
	RET Z 					;end of text if zero
	
	LD C,A					;store letter
	
	CP 29					;check if should go to new line
	JR Z,TextNewLine
	
	CP 30					;check if color change
	JR Z,TextColorChange
	
	CP 31					;check for extended symbols (32-62) coded with 31 and number 1-31
	JR Z, TextExtendedSymbols
		
TypeText2	
	PUSH HL					;paint letter attribute
	LD A,80
	SRL H
	SRL H
	SRL H
	ADD A,H
	LD H,A
	LD A,(TextColor)
	LD (HL),A
	POP HL
	
	
	
;draw a letter
	LD A,C					;restore letter code
	LD BC,Letters-8			;start of letters tiles
	
;count address of letter
	SLA A
	SLA A
	SLA A					;A=A*8 (with carry)to count adress of letter		
	
	PUSH HL
	LD HL,0
		
	LD L,A
	LD A,0
	ADC A,H	
	LD H,A					;HL=A*8	
	ADD HL,BC
	LD B,H
	LD C,L					;BC loaded with current letter tile address
	POP HL
		

	PUSH HL
	PUSH DE
	LD E,8
	
TypeText3
	LD A,(BC)				;read letter graphical data
	LD (HL),A				;and print it in screen		
	INC H
	INC BC					;go to next byte
	DEC E
	JR NZ, TypeText3
	
	POP DE
	POP HL

;go to next letter			
	INC L					;next character on the screen
	JR TypeText1
	
;-------	
TextNewLine
;go one character line down	with text
	LD HL,(TextLineAddress)			;beginning of current line in HL
	CALL OneCharacterLineDown		;count address of one line down and place it in HL
	LD (TextLineAddress),HL			;update value
	JR TypeText1					;return to letter printing 

;-------	
TextColorChange
	PUSH HL
	CALL DepackLetter				;read new color
	OR 01000000B					;(colors are bright)
	LD (TextColor),A				;and store it
	POP HL
	JR TypeText1

;-------	
TextExtendedSymbols
	PUSH HL
	CALL DepackLetter				;read next byte
	POP HL
	ADD A,31						;next sign symbol is from 32 to 62 now	
	LD C,A							;this symbol is latter read from C
	JR TypeText2


;-----------------
DepackLetter
;DE points to current byte in compressed data
;HL and B are used
	LD HL,LetterNr
	LD A,(HL)
	AND 00000011B			;A=0-2
	
	OR A					;3 bytes are packed into 2 bytes
	JR Z,DepackLetter1		;we must check if we deal now with 1st, 2nd or 3rd byte
	CP 1
	JR Z,DepackLetter2
	CP 2
	JR Z,DepackLetter3
	
	
	 
DepackLetter1			; 00001|000  01|0|00001 
						; -----
							
	LD A,(DE)				;read byte of compressed data
	AND 11111000B			
	SRL A					;rotate right three timwa
	SRL A	
	SRL A					;letter code in A
	INC (HL)				;change leter number
	RET
	
DepackLetter2			; 00001|000  01|0|00001
						;       ---  --
							
	LD A,(DE)				;read byte of compressed data
	AND 00000111B			;three bits from first compressed byte
	SLA A					;rotate left 2 times
	SLA A					
		
	LD B,A					;store in B
	INC DE					;next compressed byte
	LD A,(DE)
	AND 11000000B
	SRL A
	SRL A					
	SRL A					
	SRL A
	SRL A
	SRL A					

	OR B					;letter code in A					
	INC (HL)				;change leter number
	RET
	
DepackLetter3				; 00001|000  01|0|00001
							;      			  -----

				                					
	LD A,(DE)				;read byte of compressed data
	AND 00011111B			;letter code in A	
	INC DE					;next compressed byte			

	LD B,0
	LD (HL),B				;change leter number
	RET
	
	
;--------------
TextColor				.BYTE 0
TextLineAddress  		.WORD 0
LetterNr				.BYTE 0		;value 0-3, every 3 bytes sotre 4 letter and this number tells us which letter 

	
								
;-------------------------------										
TypeMessage
;draw message,play sound and wait for key to clear message
	;A: number of message
		
;read data from message matrix
	LD B,A
	LD HL,MessageMatrix-6	
	LD DE,6	
FindMessageStart		
	ADD HL,DE
	DJNZ FindMessageStart		;find start of message data
		
	LD B,(HL)					;read message data	
	INC HL
	LD C,(HL)					;box size in BC
	INC HL
	LD E,(HL)
	INC HL			
	LD D,(HL)					;text address in DE
	INC HL
	PUSH DE
	LD D,(HL)
	INC HL
	LD E,(HL)
	EX DE,HL					;screen position in HL
	POP DE						;restore text address in DE
								
	CALL Textbox				;type text message
	
	CALL SoundA
	;Now wait for pressing fire  clear the text
		
	
TypeMessage2	
	CALL GetPlayerInput
	BIT 4,A
	JR Z,TypeMessage2
	
	LD B,30	
TypeMessage3
	EI	
	HALT
	DI
	DJNZ TypeMessage3	
	
;refresh current location	
	CALL RefreshCurrentLocation
						
	RET
	

	
;-----------------------
Textbox
;draw a textbox (border and text inside)
	;HL: left upper text box corner
	;BC: size,B=width,C=height
	;DE: memory address of text
		
	PUSH HL
	PUSH DE
	CALL DrawBorder				;draw border around text
	POP DE
	POP HL
	CALL OneCharacterLineDown	;currrent HL points to border, text begins one line lower
	INC L						;first collumn is border,text begins in next collumn
	CALL TypeText
	RET 

;----------------
OneCharacterLineDown
	LD A,L
	ADD A,32
	LD L,A
	RET NC
	LD A,H
	ADD A,8
	LD H,A
	RET
;--------------------------------
DrawBorder
;draw an empty box with a border (for text or picture to be placed later)
	;HL: position of left upper box corner
	;BC: size,B=width,C=height
	

;clear box area	
	PUSH BC
	PUSH HL
	LD DE,EmptySpace		
	CALL TileBlock
	POP HL
	POP BC
	
DrawBorder2
;you can jump here if you don't twant to clear the inside of the box
	CALL DrawBorderA
	CALL DrawBorderB
	CALL DrawBorderC
	RET
	
;-------
DrawBorderA
;draw top edge	
	PUSH BC
	PUSH HL
	LD DE,BorderTL					;left top corner
	CALL DrawTile
	POP HL
	POP BC
	
	INC L							;horizontal line width= border width - 2 (for corners)
	DEC B
	DEC B
;top line
	PUSH BC							;store size
	LD DE,BorderHor	
DrawBorderA2
	PUSH HL
	PUSH DE
	PUSH BC
	CALL DrawTile
	POP BC
	POP DE
	POP HL
	INC L
	DJNZ DrawBorderA2
	
	LD DE,BorderTR
	PUSH HL
	CALL DrawTile					;right top corner
	POP HL
	POP BC							;restore size
	RET
	
;-------										
;draw left and right edge
DrawBorderB
	PUSH BC							;store size
	DEC C
	DEC C
	
DrawBorderB2
	CALL OneCharacterLineDown	

	LD DE,BorderVer
	PUSH HL
	PUSH BC							;draw right edge
	CALL DrawTile	
	POP BC
	POP HL
	
	PUSH HL							;store right edge
	
	LD A,L							;go to left edge
	SUB B
	DEC A
	LD L,A							
	
	LD DE,BorderVer
	PUSH BC							;draw left edge
	CALL DrawTile	
	POP BC
	
	POP HL							;restore right edge
	
	DEC C							;do it C times
	JR NZ,DrawBorderB2

	POP BC							;restore size
	RET
	
;-------										
;draw bottom edge
DrawBorderC
	CALL OneCharacterLineDown
	
	LD DE,BorderBR
	PUSH HL
	PUSH BC							;bottom right corner
	CALL DrawTile	
	POP BC
	POP HL

	DEC L							
;bottom line
	LD DE,BorderHor	
DrawBorderC2
	PUSH HL
	PUSH DE
	PUSH BC
	CALL DrawTile
	POP BC
	POP DE
	POP HL
	DEC L
	DJNZ DrawBorderC2
	
	LD DE,BorderBL					;bottom left corner
	CALL DrawTile	
	RET

;-----------
DrawDoor
;HL-corner, BC-size
;almost the same routine as drawing border but without the bottom part
	CALL DrawBorderA
	CALL DrawBorderB
	RET		
	
;-----------
;border graphics
BorderTL
	.BYTE 1,1
	.BYTE	  0,127, 64, 95, 95, 88, 91, 90
	.BYTE 65

BorderHor
	.BYTE 1,1
	.BYTE	  0,255,  0,255,255,  0,255,  0
	.BYTE 65
	
BorderTR
	.BYTE 1,1	
	.BYTE	  0,254,  2,250,250, 26,218, 90
	.BYTE 65
	
BorderVer
	.BYTE 1,1	
	.BYTE	 90, 90, 90, 90, 90, 90, 90, 90
	.BYTE 65
	
BorderBL
	.BYTE 1,1	
	.BYTE	 90, 91, 88, 95, 95, 64,127,  0
	.BYTE 65
	
BorderBR
	.BYTE 1,1	
	.BYTE	 90,218, 26,250,250,  2,254,  0
	.BYTE 65





;----------LETTERS---------------
Letters
	.BYTE	  0, 24, 60,102,102,126,102,102;a	1
	.BYTE	  0,124,102,102,124,102,102,124;b
	.BYTE	  0, 60,126, 98, 96, 98,126, 60;c
	.BYTE	  0,120,124,102,102,102,124,120;d
	.BYTE	  0,126,126, 96,124, 96,126,126;e
	.BYTE	  0,126,126, 96,124, 96, 96, 96;f
	.BYTE	  0, 60,126, 96,102, 98,126, 60;g
	.BYTE	  0,102,102,102,126,102,102,102;h
	.BYTE	  0, 24, 24, 24, 24, 24, 24, 24;i
	.BYTE	  0,  6,  6,  6,  6,102,126, 60;j
	
	.BYTE	  0,102,110,124,120,124,110,102;k	11
	.BYTE	  0, 96, 96, 96, 96, 96,126,126;l
	.BYTE	  0,198,238,254,254,214,198,198;m
	.BYTE	  0,198,230,246,254,222,206,198;n
	.BYTE	  0,124,254,198,198,198,254,124;o
	.BYTE	  0,124,126, 98, 98,124, 96, 96;p
	.BYTE	  0,124,254,198,214,214,254,124;q
	.BYTE	  0,124,126,102,102,124,110,102;r
	.BYTE	  0,124,254,192,124,  6,254,124;s
	.BYTE	  0,126,126, 24, 24, 24, 24, 24;t
	
	.BYTE	  0,102,102,102,102,102,126, 60;u	21
	.BYTE	  0,198,198,198,108,108, 56, 56;v
	.BYTE	  0,198,198,214,214,214,124, 40;w
	.BYTE	  0,198,238,124, 56,124,238,198;x
	.BYTE	  0,102,102,102, 60, 24, 24, 24;y
	.BYTE	  0,126,126, 12, 24, 48,126,126;z
	.BYTE	  0,  0,  0,  0,  0,  0,  0,  0;SPACE 27
	.BYTE	  0,  0,  0,  0,  0, 96,240, 96;. 28
	.BYTE	  0,  0,  0,  0,  0,  0,  0,  0;GO TO NEW LINE 29
	.BYTE	  0,  0,  0,  0,  0,  0,  0,  0;CHANGE COLOR 30
	.BYTE	  0,  0,  0,  0,  0,  0,  0,  0;EXTENDED SIGN PREFIX 31
	
	.BYTE	  0,  0,  0,  0, 48, 48, 32, 64;, 32 (CODE 31 1)
	.BYTE	  0, 56, 56, 48, 96, 64,  0,  0;' 33 (CODE 31 2)
	.BYTE	  0,112,248,152, 48, 96,  0, 96;? 34 (CODE 31 3)
	.BYTE	  0,  0,  0,  0,  0,  0,218,218;... 35 (CODE 31 4)
	.BYTE	  0, 96, 96, 120,96, 96,126,126; 36 (CODE 31 5)
	.BYTE	  0, 96,240,240, 96, 96,  0, 96;! 37 (CODE 31,6)
	
	.BYTE	  0,124,254,198,198,198,254,124;0 38 31,7
	.BYTE	  0, 24, 56, 56, 24, 24, 60, 60;1 39 31,8
	.BYTE	  0, 60,126, 70, 12, 56,126,126;2 40 31,9
	.BYTE	  0, 60,126,  6, 28,  6,126, 60;3 41 31,10
	.BYTE	  0, 28, 60,108,204,252, 12, 12;4 42 31,11
	.BYTE	  0,126,126, 96,124,  6,102, 60;5 43 31,12
	.BYTE	  0, 60,126, 96,124,102,102, 60;6 44 31,13
	.BYTE	  0,126,126, 12, 24, 24, 48, 48;7 45 31,14
	.BYTE	  0,124,198,198,124,198,198,124;8 46 31,15
	.BYTE	  0, 60,102,102, 62,  6,102, 60;9 47 31,16
	
	

	
	
		
